"use strict";

module.exports = function (baseUrl, user, password) {

  var rest = require('restler');
  var headers = null;
  var cookie = null;
  var userId = null;
  var attemptCount = 0;
  var hold = false;
  var baseUrl = baseUrl;
  var user = user;
  var password = password;

  var requestRaw = function (url, method, data, callback) {
    var args = [];
    for (var i = 0; i < arguments.length; i++) {
      args.push(arguments[i]);
    }

    url = baseUrl + args.shift();
    method = args.shift();
    callback = args.pop();

    if (args.length > 0) data = args.shift(); else data = {};

    var handleResponse = function (data, response) {
      if (response.statusCode >= 400) {
        if (typeof callback == 'function') {
          callback(errorForStatus(response.statusCode), data);
        } else {
          callback.processPage(errorForStatus(response.statusCode), data);
        }
        return;
      }

      if (data.length == 0) {
        data = {};
        data.path = response.headers.location;
        if (typeof data.path !== 'undefined') data.objectId = data.path.match(/[^\/]+$/)[0];

      }
      if (typeof callback == 'function') {
        callback(null, data, response);
      } else {
        callback.processPage(null, data, response);
      }
    }

    switch (method) {
      case "get":
        rest.get(url, { query: data, headers: headers }).on('complete', handleResponse);
        break;
      case "post":
        rest.post(url, { data: data, headers: headers }).on('complete', handleResponse);
        break;
      case "postJson":
        rest.postJson(url, data, { headers: headers }).on('complete', handleResponse);
        break;
      case "putJson":
        rest.putJson(url, data, { headers: headers }).on('complete', handleResponse);
        break;
      case "del":
        rest.del(url, { headers: headers }).on('complete', handleResponse);
        break;
    }
  }

  var errorForStatus = function (status) {
    switch (status) {
      case 400:
        return new Error('Error 400: Bad Request');
      case 401:
        return new Error('Error 401: Unauthorized');
      case 403:
        return new Error('Error 403: Forbidden');
      case 404:
        return new Error('Error 404: Not Found');
      case 405:
        return new Error('Error 405: Method Not Allowed');
      case 429:
        return new Error('Error 429: Too Many Request');
      case 500:
        return new Error('Error 500: Internal Server Error');
      case 502:
        return new Error('Error 502: Unknown Error');
      case 503:
        return new Error('Error 503: Service Unavailable');
      default:
        return new Error('Unexpected error ' + status + ' returned from Springboard service');
    }
  }

  var request = function (url, method, data, callback, forceRequest) {
    if (cookie || forceRequest) {
      attemptCount = 0;
      requestRaw(url, method, data, callback);
    } else {
      // Set the function to get called later if the extension is holding for auth reasons
      if (hold) setTimeout(function () { request(url, method, data, callback) }, 100);
    }
  }

  var getCookieFile = function () {
    // Retrieve auth cookie
    hold = true;
    request('auth/identity/callback', 'post', { 'auth_key': user, 'password': password }, function (err, data, response) {
      if (err) {
        console.log('Please double check your Springboard credentials and try again');
        hold = false;
        throw err;
      } else {
        cookie = response.headers['set-cookie'][0];
        headers = { 'Accept': '*/*', 'User-Agent': 'Restler for node.js', 'Cookie': cookie };
        userId = data;
        hold = false;
      }
    }, true);
  }

  getCookieFile();
  return request;
}